@@ -19,7 +19,6 @@ from msgpack import unpackb
 from ecgdetectors import Detectors
 from scipy.interpolate import interp1d
 from skimage.measure import block_reduce
-from pyndf.wt_compressor import compress, decompress, calculate_PRD
 from scipy.signal import firwin, lfilter, resample_poly, stft, butter
 
 # patch msgpack to handle numpy
@@ -231,20 +230,28 @@ def create_ndf_from_psg(data, sampling_rates, compressionbit=True, check_quality
 
 
 def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
-# Order of data(12 channels cell array of 1Xsamples PSG data): 
+# New update, we add another channel to the system which is pressure, to maintain
+# backward compatibility, a 12 channel data is still accepted. 
+# ECG must be sampled at minimum 128 Hz now, we also removed wavelet based compression previously applied earlier
+# Order of data(12 or 13 channels cell array of 1Xsamples PSG data): 
 # C3, C4, EOG-l, EOG-R, EMG-chin, EMG-left, 
-# EMG-right, ECG, Airflow, Thor, Abdo, SpO2 and Corresponding sampling rates as a numpy array
-# a new dc-block filter is added now to EEG and ECG channels, other channels are not touched
+# EMG-right, ECG, Airflow, Thor, Abdo, SpO2, Pressure and Corresponding sampling rates as a numpy array
+# a new dc-block filter is added now to EEG and ECG channels
+# we now also apply dc-block to respiratory channels but with cutoff f=0.05Hz
 
     window_128 = np.hamming(128)
     window_256 = np.hamming(256)
     window_512 = np.hamming(512)
 
-    data_exists = np.zeros(12)
+    data_exists = np.zeros(13)
     for idx, value in enumerate(data):
         if np.any(value):
             data_exists[idx] = 1  
 
+    # new data (pressure)
+    if sampling_rates.size == 12:
+        sampling_rates = np.append(sampling_rates, [25])
+
     sampling_rates[data_exists == 0] = 200
     idx = np.where(data_exists == 1)[0]
 
@@ -267,8 +274,8 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
     if np.any(sampling_rates[4:7] < 200):
         raise RuntimeError("Sampling rate must be >= 200Hz for EMG channels.")
 
-    if sampling_rates[7] < 100:
-        raise RuntimeError("Sampling rate must be >= 100Hz for ECG channels.")
+    if sampling_rates[7] < 128:
+        raise RuntimeError("Sampling rate must be >= 128Hz for ECG channels.")
 
     if np.any(sampling_rates[8:11] < 25):
         raise RuntimeError("Sampling rate must be >= 25Hz for respiratory channels")
@@ -276,9 +283,12 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
     if np.any(sampling_rates[11] < 1):
         raise RuntimeError("Sampling rate must be >= 1Hz for SpO2 channel")
 
+    if np.any(sampling_rates[12] < 25):
+        raise RuntimeError("Sampling rate must be >= 25Hz for Pressure channel")
+
     
     # this will hold the pre-processed version of the data
-    processed_data = [None]*12
+    processed_data = [None]*13
     eeg = None
     
     # apply a sharp DC block at 0.1 Hz
@@ -286,6 +296,9 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
         # check if EEG/EOG/ECG channel
         if i < 8:
             data[i] = dc_block(data[i], sampling_rates[i])
+        # check if respiration and apply a 0.05 Hz cutoff high pass
+        if (i >= 8 and i <=10) or i == 12:
+            data[i] = dc_block(data[i], sampling_rates[i], 0.05)
 
     # apply filters to first 5 channels as per spec
     # Order 50 FIR filter
@@ -339,18 +352,33 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
             processed_data[i] = resample_poly(processed_data[i], P, Q)
 
     # Process ECG
-    if data_exists[7] and sampling_rates[7] != 100:
-        P = 100
-        Q = sampling_rates[7]
-        data[7] = resample_poly(data[7], P, Q)
-        processed_data[7] = compress(data[7])
+    if data_exists[7]:
+        if sampling_rates[7] != 128:
+            P = 128
+            Q = sampling_rates[7]
+            processed_data[7] = resample_poly(data[7], P, Q)
+        else:
+            processed_data[7] = data[7]
         
     # Process Respiration
     for i in range(8,12):
-        if data_exists[i] and sampling_rates[i] != 25:
+        if data_exists[i]:
+            if sampling_rates[i] != 25:
+                P = 25
+                Q = sampling_rates[i]
+                processed_data[i] = resample_poly(data[i], P, Q)
+            else:
+                processed_data[i] = data[i]
+
+    # Pressure
+    if data_exists[12]:
+        if sampling_rates[12] != 25:
             P = 25
-            Q = sampling_rates[i]
-            processed_data[i] = resample_poly(data[i], P, Q)
+            Q = sampling_rates[12]
+            processed_data[12] = resample_poly(data[12], P, Q)
+        else:
+            processed_data[12] = data[12] 
+
 
     # Process EEG and chin EMG for respiration
     # only if respiratory channels are present
@@ -438,8 +466,14 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
     if np.sum(data_exists[5:7]) == 0:
         print("\u001b[31;1mWARNING:\u001b[0m Leg EMG not found, PLM detection disabled.")
 
-    if np.sum(data_exists[8:11]) == 0:
-        print("\u001b[31;1mWARNING:\u001b[0m No respiratory channel found, respiratory events detection disabled.")
+    if data_exists[8] == 0:
+        print("\u001b[31;1mWARNING:\u001b[0m Airflow channel not detected, respiratory events detection disabled.")
+
+    if np.sum(data_exists[9:11]) == 0:
+        print("\u001b[31;1mWARNING:\u001b[0m Thor/Abdo not detected, apnea events won't be sub-classified.")
+    
+    if data_exists[12] == 0:
+        print("\u001b[31;1mWARNING:\u001b[0m Pressure channel not detected, respiratory events detection disabled.")
 
     spec = {}
     
@@ -490,6 +524,10 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
     if data_exists[11]:    
         spec['spo2'] = processed_data[11]
 
+    # pressure
+    if data_exists[12]:    
+        spec['pressure'] = processed_data[12]
+
     
     payload = msgpack.packb(spec, use_single_float=True, use_bin_type=True) 
     shaHash = hashlib.sha1()
@@ -500,7 +538,7 @@ def create_ndf(data, sampling_rates, compressionbit=True, check_quality = True):
     urlSafeHash = base64.urlsafe_b64encode(("".join(map(chr, msgDigest))).encode('UTF-8')).decode('ascii')
     urlSafeHash = urlSafeHash[0:-1]
 
-    header = '{"version":"PSG", "duration": '+ str(int(totalEpochs*30)) + ', "url":"' + urlSafeHash + '", "spec":' + json.dumps(list(spec.keys())) + '}'
+    header = '{"version":"PSG", "revision": "1", "duration": '+ str(int(totalEpochs*30)) + ', "url":"' + urlSafeHash + '", "spec":' + json.dumps(list(spec.keys())) + '}'
     header = '{:<256}'.format(header[:256])
     
     signature = bytearray(struct.pack('<3s?', b'NDF', compressionbit))  
